package com.bull.eclipse.jonas.actions.ejb;

import java.io.File;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IImportDeclaration;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.jdt.internal.core.PackageFragment;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.preference.PreferenceContentProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.CheckedTreeSelectionDialog;
import org.eclipse.ui.dialogs.ListSelectionDialog;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasPluginResources;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.actions.JonasPackageAbstractActionDelegate;
import com.bull.eclipse.jonas.utils.AntUtils;
import com.bull.eclipse.jonas.utils.EjbManagement;
import com.bull.eclipse.jonas.utils.UpdatePropsFile;
import com.bull.eclipse.jonas.utils.xml.FileXmlUtils;
import com.bull.eclipse.jonas.utils.xml.UpdateXmlFile;
import com.bull.eclipse.jonas.utils.xml.XMLUtils;
import com.bull.eclipse.newbean.FacadeEJBGenerate;
import com.bull.eclipse.newbean.EntityDescValue;
import com.bull.eclipse.newbean.MethodDescValue;
import com.bull.eclipse.newbean.NewFacadeValue;



public class AggregateFacadeEjbActionDelegate extends JonasPackageAbstractActionDelegate
{
	static private Object[] oldSelection = null; 
	static private String oldBeanName = null;
	
	public boolean doActionOn(IPackageFragment pack) throws Exception {
		// Update build.properties with JONAS_ROOT
		if (pack.containsJavaResources()) 
			throw new Exception("You must execute this function on a deployment unit !");
		
		JonasProject jp = (JonasProject)pack.getJavaProject().getProject().getNature(JonasLauncherPlugin.NATURE_ID);
		UpdatePropsFile.updatePropsProject(jp);		
		
		
		String projDir = jp.getProject().getLocation().toOSString();		
		String packName = pack.getPath().removeFirstSegments(1).toOSString();	
		File ejb_standard_xml = new File(projDir , pack.getPath().removeFirstSegments(1).append("ejb-jar.xml").toOSString());
		if (!ejb_standard_xml.exists())
			throw new Exception("The deployment unit must be create by the middlegen functionnality");
				
		Hashtable hashtable = FileXmlUtils.getAllNameAndPackageEJBEntity(ejb_standard_xml);
		
		// Entities EJB name
		Vector vec = new Vector(hashtable.keySet());
		Vector entitiesChoice =  ejbEntitiesNameChoice(vec);
		Enumeration nameEnum = entitiesChoice.elements();
		Vector entitesChoicePackage = new Vector();
		while (nameEnum.hasMoreElements()) {
			entitesChoicePackage.add(hashtable.get(nameEnum.nextElement()));
		}
		Vector vecValue = new Vector();
		for (int i = 0; i < entitesChoicePackage.size(); i++) {
			EntityDescValue napv = new EntityDescValue();
			napv.setBeanName((String) entitiesChoice.get(i));
			napv.setPackageName((String) entitesChoicePackage.get(i));
			vecValue.add(napv);
		}
		
		vecValue = getMethods(vecValue,pack.getJavaProject(),jp);
		
		File tempFile = new File(jp.getProject().getLocation().append("temp").toOSString());
		if (!tempFile.exists())
    		tempFile.mkdirs();
		
		
		FacadeEJBGenerate feg = new FacadeEJBGenerate(jp);
		NewFacadeValue nfv = new NewFacadeValue();
		nfv.setVecValue(vecValue);
		String beanName = getChoice(oldBeanName,"Facade EJB name","What is the facade ejb name ?");
		oldBeanName = beanName;
		String firstChar = beanName.substring(0,1);
		String endString = beanName.substring(1,beanName.length());
		nfv.setBeanName(firstChar.toUpperCase().concat(endString));
		nfv.setPackageName(pack.getPath().removeFirstSegments(2).toOSString().replace(File.separatorChar, '.').concat(".session.").concat(firstChar.toLowerCase().concat(endString)));
		feg.runNewFacade(nfv);
		// Update standard deployment descriptor
		UpdateXmlFile.updateEjbStandardXmlFile(ejb_standard_xml, new File(tempFile, "facadeNode.xml"));
		
		
		jp.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );			 
		
		
		
	    return false;
	}			

	private Vector ejbEntitiesNameChoice(Vector vecEjbName) throws Exception {
		final String[] listEJBEntity = (String[]) vecEjbName.toArray(new String[0]);
		ListSelectionDialog lsd =
			new ListSelectionDialog(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
				new Object(),
				new IStructuredContentProvider() {

			public void dispose() {
			}

			public Object[] getElements(Object inputElement) {
				return listEJBEntity;
			}

			public void inputChanged(Viewer viewer1, Object obj, Object obj1) {
				int i = 0;
			}

		}, new LabelProvider(), "Select entities beans you want to access by the ejb session");
		
		lsd.setTitle("Select j2ee modules");
		if (oldSelection == null) {
			lsd.setInitialSelections(listEJBEntity);	
		} else {
			lsd.setInitialSelections(oldSelection);
		}
		
		lsd.open();

		Object[] results_upd = lsd.getResult();
		
		oldSelection = results_upd;
		
		int ret = lsd.getReturnCode();
		// return null ( list of methods) , if user press cancel or nothing is selected 
		if (ret == 1)
			throw new Exception("Operation is cancelled");
		if ((results_upd == null) || (results_upd.length == 0))
			throw new Exception("There is no element to build the application");
		
		Vector entityChoice = new Vector();
		for(int i=0;i<oldSelection.length;i++) {
			entityChoice.add(oldSelection[i]);
		}

		return entityChoice;
	}
	
	private String getChoice(String defaultValue, String title_label, String label) {
		InputDialog id = new InputDialog(JavaPlugin.getActiveWorkbenchShell(),
				title_label,
				label, defaultValue, null);

		if (id.open() == org.eclipse.jface.window.Window.OK)
			return id.getValue();
		else
			return null;

	}

	protected static Vector getMethods(Vector vecValue, IJavaProject javp, JonasProject jp) throws JavaModelException {
		IPath ejbPath = jp.getProject().getLocation().append(jp.getSrcEJBDir());
		Vector napvVec = new Vector();
		for (int i=0; i < vecValue.size(); i++) {
			EntityDescValue napv = (EntityDescValue) vecValue.elementAt(i);
			IPath packFragPath = ejbPath.append(napv.getPackageName().replace('.',File.separatorChar));
			IPath classes = packFragPath.append(napv.getBeanName().concat("Local.java"));
			String className = napv.getPackageName().concat(".").concat(napv.getBeanName()).concat("Local");
			napv.setMethodDesc(seekMethod(napv, className, javp));
			classes = packFragPath.append(napv.getBeanName().concat("LocalHome.java"));
			className = napv.getPackageName().concat(".").concat(napv.getBeanName()).concat("LocalHome");
			//TODO HOME is necessary ?? napv.setMethodHomeDesc(seekMethod(napv, className, javp));

			napvVec.add(napv);
		}
		return napvVec;

	}
	
	private static Vector seekMethod(EntityDescValue napv, String className, IJavaProject japv) throws JavaModelException {
		IType ty = japv.findType(className);
		IMethod[] ime = ty.getMethods();
		Vector vecSignature = new Vector();
		for(int j=0; j<ime.length; j++) {
			MethodDescValue mdv = new MethodDescValue();
			String signature = Signature.toString(ime[j].getSignature(),ime[j].getElementName(),ime[j].getParameterNames(),true,true);
			String[] exceptionListe = ime[j].getExceptionTypes();
			String exception = " throws ";
			for (int g=0; g<exceptionListe.length; g++) {
				if (!exception.equals(" throws "))
					exception = exception.concat(Signature.toString(exceptionListe[g])).concat(", ");
				else
					exception = exception.concat(Signature.toString(exceptionListe[g])).concat(", ");
			}
			mdv.setSignature(signature.concat(exception));
			mdv.setName(ime[j].getElementName());
			String[] parameterNameList = ime[j].getParameterNames();
			String parameterName = "";
			for (int k=0; k<parameterNameList.length; k++) {
				if (parameterName.equals(""))
					parameterName = parameterNameList[k];
				else
					parameterName = ", ".concat(parameterName);
			}

			mdv.setParameterName(parameterName);
			vecSignature.add(mdv);				
		}
			
		return vecSignature;
		
	}
}

